/*****************************************************************************
 *
 * CHECK_REAL.C
 *
 * Program: RealMedia plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Pedro Leite (leite@cic.ua.pt)
 *
 * Based on CHECK_HTTP.C
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: 25-10-1999
 *
 * Command line: CHECK_REAL <host_address> [-e expect] [-u url] [-p port]
 *                          [-hn host_name] [-wt warn_time] [-ct crit_time]
 *                          [-to to_sec]
 *
 * Description:
 *
 * This plugin will attempt to open an RTSP connection with the host.
 * Successul connects return STATE_OK, refusals and timeouts return
 * STATE_CRITICAL, other errors return STATE_UNKNOWN.  Successful connects,
 * but incorrect reponse messages from the host result in STATE_WARNING return
 * values.  If you are checking a virtual server that uses "host headers"you
 * must supply the FQDN (fully qualified domain name) as the [host_name]
 * argument.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netutils.h"

#define REAL_PORT	554
#define REAL_EXPECT	"RTSP/1."
#define REAL_URL	""

time_t start_time, end_time;
int server_port = REAL_PORT;
char server_address[MAX_HOST_ADDRESS_LENGTH];
char host_name[MAX_HOST_ADDRESS_LENGTH];
char server_url[MAX_INPUT_BUFFER] = REAL_URL;
char server_expect[MAX_INPUT_BUFFER] = REAL_EXPECT;
int socket_timeout = DEFAULT_SOCKET_TIMEOUT;
int warning_time = 0;
int check_warning_time = FALSE;
int critical_time = 0;
int check_critical_time = FALSE;
char user_auth[MAX_INPUT_BUFFER] = "";

int process_arguments (int, char **);
static char *base64 (char *bin, int len);


int
main (int argc, char **argv)
{
  int sd;
  int result;
  char buffer[MAX_INPUT_BUFFER];
  char *status_line;
  char *auth;

  result = process_arguments (argc, argv);

  if (result != OK)
    {

      printf ("Incorrect number of arguments supplied\n");
      printf ("\n");
      printf ("REAL plugin for NetSaint\n");
      printf ("Copyright (c) 1999 Pedro Leite (leite@cic.ua.pt)\n");
      printf ("Last Modified: 25-10-1999\n");
      printf ("License: GPL\n");
      printf ("\n");
      printf ("Usage: %s <host_address> [-e expect] [-u url] [-p port] [-hn host_name] [-wt warn_time]\n",argv[0]);
      printf("        [-ct crit_time] [-to to_sec] [-a auth]\n");
      printf ("\n");
      printf ("Options:\n");
      printf (" [expect]    = String to expect in first line of server response - default is \"%s\"\n", REAL_EXPECT);
      printf (" [url]       = Optional URL to GET - default is root document\n");
      printf (" [port]      = Optional port number to use - default is %d\n", REAL_PORT);
      printf (" [host_name] = Optional host name argument to GET command - used for servers using host headers\n");
      printf (" [warn_time] = Response time in seconds necessary to result in a warning status\n");
      printf (" [crit_time] = Response time in seconds necessary to result in a critical status\n");
      printf (" [to_sec]    = Number of seconds before connection attempt times out - default is %d seconds\n", DEFAULT_SOCKET_TIMEOUT);
      printf (" [auth]      = Optional username:password for sites requiring basic authentication\n");
      printf ("\n");
      printf ("This plugin attempts to contact the REAL service on the specified host.\n");
      printf ("If possible, supply an IP address for the host address, as this will bypass the DNS lookup.\n");
      printf ("\n");

      return STATE_UNKNOWN;
    }

  /* initialize alarm signal handling */
  signal (SIGALRM, socket_timeout_alarm_handler);

  /* set socket timeout */
  alarm (socket_timeout);

  /* try to connect to the host at the given port number */
  time (&start_time);
  result = my_tcp_connect (server_address, server_port, &sd);
  /* we connected, so close connection before exiting */
  if (result == STATE_OK)
    {

      /* Part I - Server Check */

      /* send the OPTIONS request */
      sprintf (buffer, "OPTIONS rtsp://%s:%d RTSP/1.0\n",host_name,server_port);
      result = send (sd, buffer, strlen (buffer), 0);

      /* send the header sync */
      sprintf (buffer, "CSeq: 1\n", host_name, server_port);
      result = send (sd, buffer, strlen (buffer), 0);

      /* send a newline so the server knows we're done with the request */
      sprintf (buffer, "\n");
      result = send (sd, buffer, strlen (buffer), 0);

      /* watch for the REAL connection string */
      result = recv (sd, buffer, MAX_INPUT_BUFFER - 1, 0);

      /* return a CRITICAL status if we couldn't read any data */
      if (result == -1)
	{
	  printf ("No data received from host\n");
	  result = STATE_CRITICAL;
	}
      else
	{
	  /* make sure we find the response we are looking for */
	  if (!strstr (buffer, REAL_EXPECT))
	    {
	      if (server_port == REAL_PORT)
		printf ("Invalid REAL response received from host\n");
	      else
		printf ("Invalid REAL response received from host on port %d\n",server_port);
	    }
	  else
	    {
	      /* else we got the REAL string, so check the return code */

	      time (&end_time);

	      result = STATE_OK;

	      status_line = (char *) strtok (buffer, "\n");

	      if (strstr (status_line, "200"))
		result = STATE_OK;

	      /* client errors result in a warning state */
	      else if (strstr (status_line, "400"))
		result = STATE_WARNING;
	      else if (strstr (status_line, "401"))
		result = STATE_WARNING;
	      else if (strstr (status_line, "402"))
		result = STATE_WARNING;
	      else if (strstr (status_line, "403"))
		result = STATE_WARNING;
	      else if (strstr (status_line, "404"))
		result = STATE_WARNING;

	      /* server errors result in a critical state */
	      else if (strstr (status_line, "500"))
		result = STATE_CRITICAL;
	      else if (strstr (status_line, "501"))
		result = STATE_CRITICAL;
	      else if (strstr (status_line, "502"))
		result = STATE_CRITICAL;
	      else if (strstr (status_line, "503"))
		result = STATE_CRITICAL;

	      else
		result = STATE_UNKNOWN;
	    }
	}

      /* Part II - Check stream exists and is ok */
      if ((result == STATE_OK) && (strcmp(server_url,"")))
	{

	  /* Part I - Server Check */

	  /* send the OPTIONS request */
	  sprintf (buffer, "DESCRIBE rtsp://%s:%d%s RTSP/1.0\n", host_name, server_port, server_url);
	  result = send (sd, buffer, strlen (buffer), 0);

	  /* send the header sync */
	  sprintf (buffer, "CSeq: 2\n", host_name, server_port);
	  result = send (sd, buffer, strlen (buffer), 0);

	  /* send a newline so the server knows we're done with the request */
	  sprintf (buffer, "\n");
	  result = send (sd, buffer, strlen (buffer), 0);

	  /* watch for the REAL connection string */
	  result = recv (sd, buffer, MAX_INPUT_BUFFER - 1, 0);

	  /* return a CRITICAL status if we couldn't read any data */
	  if (result == -1)
	    {
	      printf ("No data received from host\n");
	      result = STATE_CRITICAL;
	    }
	  else
	    {
	      /* make sure we find the response we are looking for */
	      if (!strstr (buffer, REAL_EXPECT))
		{
		  if (server_port == REAL_PORT)
		    printf ("Invalid REAL response received from host\n");
		  else
		    printf ("Invalid REAL response received from host on port %d\n", server_port);
		}
	      else
		{

		  /* else we got the REAL string, so check the return code */

		  time (&end_time);

		  result = STATE_OK;

		  status_line = (char *) strtok (buffer, "\n");

		  if (strstr (status_line, "200"))
		    result = STATE_OK;

		  /* client errors result in a warning state */
		  else if (strstr (status_line, "400"))
		    result = STATE_WARNING;
		  else if (strstr (status_line, "401"))
		    result = STATE_WARNING;
		  else if (strstr (status_line, "402"))
		    result = STATE_WARNING;
		  else if (strstr (status_line, "403"))
		    result = STATE_WARNING;
		  else if (strstr (status_line, "404"))
		    result = STATE_WARNING;

		  /* server errors result in a critical state */
		  else if (strstr (status_line, "500"))
		    result = STATE_CRITICAL;
		  else if (strstr (status_line, "501"))
		    result = STATE_CRITICAL;
		  else if (strstr (status_line, "502"))
		    result = STATE_CRITICAL;
		  else if (strstr (status_line, "503"))
		    result = STATE_CRITICAL;

		  else
		    result = STATE_UNKNOWN;
		}
	    }
	}

      /* Return results */
      if (result == STATE_OK)
	{

	  if (check_critical_time == TRUE && (end_time - start_time) > critical_time)
	    result = STATE_CRITICAL;
	  else if (check_warning_time == TRUE && (end_time - start_time) > warning_time)
	    result = STATE_WARNING;

	  /* Put some HTML in here to create a dynamic link */
	  printf ("REAL %s - %d second response time\n", (result == STATE_OK) ? "ok" : "problem", (int) (end_time - start_time));
	}
      else
	printf ("%s\n", status_line);

    }

  /* close the connection */
  close (sd);

  /* reset the alarm */
  alarm (0);

  return result;
}


/* process command-line arguments */
int
process_arguments (int argc, char **argv)
{
  int x;

  /* no options were supplied */
  if (argc < 2)
    return ERROR;

  /* first option is always the server name/address */
  strncpy (server_address, argv[1], sizeof (server_address) - 1);
  server_address[sizeof (server_address) - 1] = '\x0';

  /* set the host name to the server address (until its overridden) */
  strcpy (host_name, server_address);

  /* process all remaining arguments */
  for (x = 3; x <= argc; x++)
    {

      /* we got the string to expect from the server */
      if (!strcmp (argv[x - 1], "-e"))
	{
	  if (x < argc)
	    {
	      strncpy (server_expect, argv[x], sizeof (server_expect) - 1);
	      server_expect[sizeof (server_expect) - 1] = '\x0';
	      x++;
	    }
	  else
	    return ERROR;
	}

      /* we got the URL to check */
      else if (!strcmp (argv[x - 1], "-u"))
	{
	  if (x < argc)
	    {
	      strncpy (server_url, argv[x], sizeof (server_url) - 1);
	      server_url[sizeof (server_url) - 1] = '\x0';
	      x++;
	    }
	  else
	    return ERROR;
	}

      /* we go the host name to use in the host header */
      else if (!strcmp (argv[x - 1], "-hn"))
	{
	  if (x < argc)
	    {
	      strncpy (host_name, argv[x], sizeof (host_name) - 1);
	      host_name[sizeof (host_name) - 1] = '\x0';
	      x++;
	    }
	  else
	    return ERROR;
	}

      /* we got the port number to use */
      else if (!strcmp (argv[x - 1], "-p"))
	{
	  if (x < argc)
	    {
	      server_port = atoi (argv[x]);
	      x++;
	    }
	  else
	    return ERROR;
	}

      /* we got the socket timeout */
      else if (!strcmp (argv[x - 1], "-to"))
	{
	  if (x < argc)
	    {
	      socket_timeout = atoi (argv[x]);
	      if (socket_timeout <= 0)
		return ERROR;
	      x++;
	    }
	  else
	    return ERROR;
	}

      /* we got the warning threshold time */
      else if (!strcmp (argv[x - 1], "-wt"))
	{
	  if (x < argc)
	    {
	      warning_time = atoi (argv[x]);
	      check_warning_time = TRUE;
	      x++;
	    }
	  else
	    return ERROR;
	}

      /* we got the critical threshold time */
      else if (!strcmp (argv[x - 1], "-ct"))
	{
	  if (x < argc)
	    {
	      critical_time = atoi (argv[x]);
	      check_critical_time = TRUE;
	      x++;
	    }
	  else
	    return ERROR;
	}

      /* we got the user's authorization info */
      else if (!strcmp (argv[x - 1], "-a"))
	{
	  if (x < argc)
	    {
	      strncpy (user_auth, argv[x], sizeof (user_auth) - 1);
	      user_auth[sizeof (user_auth) - 1] = '\x0';
	      x++;
	    }
	  else
	    return ERROR;
	}

      /* else we got something else... */
      else
	return ERROR;
    }

  return OK;
}



/* written by lauri alanko */
static char *
base64 (char *bin, int len)
{

  char *buf = (char *) malloc ((len + 2) / 3 * 4 + 1);
  int i = 0, j = 0;

  char BASE64_END = '=';
  char base64_table[64] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
  "abcdefghijklmnopqrstuvwxyz"
  "0123456789+/";

  while (j < len - 2)
    {
      buf[i++] = base64_table[bin[j] >> 2];
      buf[i++] = base64_table[((bin[j] & 3) << 4) | (bin[j + 1] >> 4)];
      buf[i++] = base64_table[((bin[j + 1] & 15) << 2) | (bin[j + 2] >> 6)];
      buf[i++] = base64_table[bin[j + 2] & 63];
      j += 3;
    }

  switch (len - j)
    {
    case 1:
      buf[i++] = base64_table[bin[j] >> 2];
      buf[i++] = base64_table[(bin[j] & 3) << 4];
      buf[i++] = BASE64_END;
      buf[i++] = BASE64_END;
      break;
    case 2:
      buf[i++] = base64_table[bin[j] >> 2];
      buf[i++] = base64_table[((bin[j] & 3) << 4)
			      | (bin[j + 1] >> 4)];
      buf[i++] = base64_table[(bin[j + 1] & 15) << 2];
      buf[i++] = BASE64_END;
      break;
    case 0:
      break;
    }

  buf[i] = '\0';
  return buf;
}
